import {extends as $3whmL$extends} from "@swc/helpers";
import {h as $3whmL$h} from "preact";
import {useState as $3whmL$useState, useRef as $3whmL$useRef, useEffect as $3whmL$useEffect, useCallback as $3whmL$useCallback} from "preact/hooks";
import {memo as $3whmL$memo} from "preact/compat";
import {EditorView as $3whmL$EditorView} from "@codemirror/view";
import {StateField as $3whmL$StateField, EditorState as $3whmL$EditorState, StateEffect as $3whmL$StateEffect} from "@codemirror/state";
import {setDiagnostics as $3whmL$setDiagnostics} from "@codemirror/lint";








const $b5e4219d59abbb5aa4500ad182a0bc8c$var$setReadOnlyEffect = $3whmL$StateEffect.define();
const $b5e4219d59abbb5aa4500ad182a0bc8c$var$readOnlyState = $3whmL$StateField.define({
    create () {
        return true;
    },
    update (value, tr) {
        for (let effect of tr.effects)if (effect.is($b5e4219d59abbb5aa4500ad182a0bc8c$var$setReadOnlyEffect)) value = effect.value;
        return value;
    }
});
function $b5e4219d59abbb5aa4500ad182a0bc8c$export$a9c1fcb7eeda2143(value, extensions) {
    return $3whmL$EditorState.create({
        doc: value,
        extensions: [
            ...extensions,
            $b5e4219d59abbb5aa4500ad182a0bc8c$var$readOnlyState,
            $3whmL$EditorState.changeFilter.of(({ startState: startState  })=>!startState.field($b5e4219d59abbb5aa4500ad182a0bc8c$var$readOnlyState, false)
            ), 
        ]
    });
}
const $b5e4219d59abbb5aa4500ad182a0bc8c$var$emittedStates = new WeakSet();
const $b5e4219d59abbb5aa4500ad182a0bc8c$export$63d6997c0a04280f = $3whmL$memo(function Codemirror({ state: state , onChange: onChange = null , readOnly: readOnly = false , diagnostics: diagnostics = null , ...rest }) {
    if (!state) throw new Error("Missing state!");
    const container = $3whmL$useRef(null);
    const view = $3whmL$useRef(null);
    const dispatchRefs = $3whmL$useRef({
    });
    $3whmL$useEffect(()=>{
        dispatchRefs.current.onChange = onChange;
    }, [
        onChange
    ]);
    // --- State ---
    $3whmL$useEffect(()=>{
        // (subsequent render)
        if (!view.current) return;
        if ($b5e4219d59abbb5aa4500ad182a0bc8c$var$emittedStates.has(state)) return;
        view.current.setState(state);
    }, [
        state
    ]);
    $3whmL$useEffect(()=>{
        // (first render)
        let v = view.current = new $3whmL$EditorView({
            state: state,
            dispatch: (t)=>{
                const { onChange: onChange1  } = dispatchRefs.current;
                v.update([
                    t
                ]);
                if (onChange1) {
                    $b5e4219d59abbb5aa4500ad182a0bc8c$var$emittedStates.add(v.state);
                    onChange1(v.state, t);
                }
            }
        });
        container.current.appendChild(v.dom);
        return ()=>v.destroy()
        ;
    }, []);
    // --- readOnly ---
    $3whmL$useEffect(()=>{
        // if readOnly changes (first and subsequent render)
        view.current.dispatch({
            effects: $b5e4219d59abbb5aa4500ad182a0bc8c$var$setReadOnlyEffect.of(readOnly)
        });
    }, [
        readOnly
    ]);
    $3whmL$useEffect(()=>{
        // reapply readOnly if the state was changed externally
        if ($b5e4219d59abbb5aa4500ad182a0bc8c$var$emittedStates.has(state)) return;
        view.current.dispatch({
            effects: $b5e4219d59abbb5aa4500ad182a0bc8c$var$setReadOnlyEffect.of(readOnly)
        });
    }, [
        readOnly,
        state
    ]);
    // --- Diagnostics ---
    $3whmL$useEffect(()=>{
        // if diagnostics changes (first and subsequent render)
        view.current.dispatch($3whmL$setDiagnostics(view.current.state, diagnostics && diagnostics.length > 0 ? diagnostics : []));
    }, [
        diagnostics
    ]);
    $3whmL$useEffect(()=>{
        // reapply diagnostics if the state was changed externally
        if ($b5e4219d59abbb5aa4500ad182a0bc8c$var$emittedStates.has(state)) return;
        view.current.dispatch($3whmL$setDiagnostics(view.current.state, diagnostics && diagnostics.length > 0 ? diagnostics : []));
    }, [
        diagnostics,
        state
    ]);
    return(/*#__PURE__*/ $3whmL$h("div", $3whmL$extends({
        ref: container
    }, rest)));
});
function $b5e4219d59abbb5aa4500ad182a0bc8c$export$3e552b24c5ce7ce2({ value: value , onChange: _onChange , readOnly: readOnly , diagnostics: diagnostics , extensions: extensions ,  }) {
    const [state, setState] = $3whmL$useState($b5e4219d59abbb5aa4500ad182a0bc8c$export$a9c1fcb7eeda2143(value, extensions));
    $3whmL$useEffect(()=>{
        if (value !== state.doc.toString()) setState($b5e4219d59abbb5aa4500ad182a0bc8c$export$a9c1fcb7eeda2143(value, extensions));
    }, [
        value
    ]);
    $3whmL$useEffect(()=>{
        setState($b5e4219d59abbb5aa4500ad182a0bc8c$export$a9c1fcb7eeda2143(value, extensions));
    }, [
        extensions
    ]);
    const onChange = $3whmL$useCallback((newState, transaction)=>{
        setState(newState);
        if (!transaction.changes.empty) _onChange(newState.doc.toString());
    }, []);
    return(/*#__PURE__*/ $3whmL$h($b5e4219d59abbb5aa4500ad182a0bc8c$export$63d6997c0a04280f, {
        state: state,
        onChange: onChange,
        readOnly: readOnly,
        diagnostics: diagnostics,
        class: "editor"
    }));
}


export {$b5e4219d59abbb5aa4500ad182a0bc8c$export$3e552b24c5ce7ce2 as CodemirrorEditor, $b5e4219d59abbb5aa4500ad182a0bc8c$export$a9c1fcb7eeda2143 as createState, $b5e4219d59abbb5aa4500ad182a0bc8c$export$63d6997c0a04280f as Codemirror};
//# sourceMappingURL=index.es.js.map
